Index: channel.c
===================================================================
RCS file: /usr/cvsroot/asterisk/channel.c,v
retrieving revision 1.184
diff -u -r1.184 channel.c
--- channel.c	31 Mar 2005 19:07:27 -0000	1.184
+++ channel.c	1 Apr 2005 09:04:06 -0000
@@ -39,6 +39,7 @@
 #include <asterisk/utils.h>
 #include <asterisk/lock.h>
 #include <asterisk/app.h>
+#include <libpri.h>
 #ifdef ZAPTEL_OPTIMIZATIONS
 #include <sys/ioctl.h>
 #ifdef __linux__
@@ -243,6 +244,25 @@
 	}
 }
 
+char *ast_transfercapability2str(int transfercapability)
+{
+	switch(transfercapability) {
+	case PRI_TRANS_CAP_SPEECH:
+		return "SPEECH";
+	case PRI_TRANS_CAP_DIGITAL:
+		return "DIGITAL";
+	case PRI_TRANS_CAP_RESTRICTED_DIGITAL:
+		return "RESTRICTED_DIGITAL";
+	case PRI_TRANS_CAP_3_1K_AUDIO:
+		return "3K1AUDIO";
+	case PRI_TRANS_CAP_DIGITAL_W_TONES:
+		return "DIGITAL_W_TONES";
+	case PRI_TRANS_CAP_VIDEO:
+		return "VIDEO";
+	default:
+		return "UNKNOWN";
+	}
+}
 
 int ast_best_codec(int fmts)
 {
Index: apps/Makefile
===================================================================
RCS file: /usr/cvsroot/asterisk/apps/Makefile,v
retrieving revision 1.96
diff -u -r1.96 Makefile
--- apps/Makefile	27 Mar 2005 22:39:16 -0000	1.96
+++ apps/Makefile	1 Apr 2005 09:04:17 -0000
@@ -31,7 +31,7 @@
      app_talkdetect.so app_alarmreceiver.so app_userevent.so app_verbose.so \
      app_test.so app_forkcdr.so app_math.so app_realtime.so \
      app_dumpchan.so app_waitforsilence.so app_while.so app_setrdnis.so \
-     app_md5.so app_readfile.so app_chanspy.so
+     app_md5.so app_readfile.so app_chanspy.so app_settransfercapability.so
 
 ifneq (${OSARCH},Darwin)
 ifneq (${OSARCH},SunOS)
Index: apps/app_dial.c
===================================================================
RCS file: /usr/cvsroot/asterisk/apps/app_dial.c,v
retrieving revision 1.143
diff -u -r1.143 app_dial.c
--- apps/app_dial.c	17 Mar 2005 22:39:04 -0000	1.143
+++ apps/app_dial.c	1 Apr 2005 09:04:20 -0000
@@ -77,8 +77,8 @@
 "      'r' -- indicate ringing to the calling party, pass no audio until answered.\n"
 "      'm[(class)]' -- provide hold music to the calling party until answered (optionally\n"
 "                      with the specified class.\n"
-"      'M(x[^arg]) -- Executes the macro (x with ^ delim arg list) upon connect of the call.\n"
-"                     Also, the macro can set the MACRO_RESULT variable to do the following:\n"
+"      'M(x[^arg])' -- Executes the macro (x with ^ delim arg list) upon connect of the call.\n"
+"                      Also, the macro can set the MACRO_RESULT variable to do the following:\n"
 "                     -- ABORT - Hangup both legs of the call.\n"
 "                     -- CONGESTION - Behave as if line congestion was encountered.\n"
 "                     -- BUSY - Behave as if a busy signal was encountered. (n+101)\n"
@@ -1039,8 +1039,10 @@
 		tmp->chan->cid.cid_tns = chan->cid.cid_tns;
 		/* Presense of ADSI CPE on outgoing channel follows ours */
 		tmp->chan->adsicpe = chan->adsicpe;
-		/* pass the digital flag */
+		/* Pass the digital flag */
 		ast_copy_flags(tmp->chan, chan, AST_FLAG_DIGITAL);
+		/* Pass the transfer capability */
+		tmp->chan->transfercapability = chan->transfercapability;
 
 		/* If we have an outbound group, set this peer channel to it */
 		if (outbound_group)
Index: channels/chan_zap.c
===================================================================
RCS file: /usr/cvsroot/asterisk/channels/chan_zap.c,v
retrieving revision 1.422
diff -u -r1.422 chan_zap.c
--- channels/chan_zap.c	28 Mar 2005 20:48:24 -0000	1.422
+++ channels/chan_zap.c	1 Apr 2005 09:05:10 -0000
@@ -1869,10 +1869,12 @@
 		p->digital = ast_test_flag(ast,AST_FLAG_DIGITAL);
 		pri_sr_set_channel(sr, p->bearer ? PVT_TO_CHANNEL(p->bearer) : PVT_TO_CHANNEL(p), 
 								p->pri->nodetype == PRI_NETWORK ? 0 : 1, 1);
-		pri_sr_set_bearer(sr, p->digital ? PRI_TRANS_CAP_DIGITAL : PRI_TRANS_CAP_SPEECH, 
+		pri_sr_set_bearer(sr, p->digital ? PRI_TRANS_CAP_DIGITAL : ast->transfercapability, 
 					(p->digital ? -1 : 
 						((p->law == ZT_LAW_ALAW) ? PRI_LAYER_1_ALAW : PRI_LAYER_1_ULAW)));
- 		dp_strip = 0;
+		if (option_verbose > 2)
+			ast_verbose(VERBOSE_PREFIX_3 "Requested transfer capability: 0x%.2x - %s\n", ast->transfercapability, ast_transfercapability2str(ast->transfercapability));
+		dp_strip = 0;
  		pridialplan = p->pri->dialplan - 1;
  		if (pridialplan == -2) { /* compute dynamically */
  			if (strncmp(c + p->stripmsd, p->pri->internationalprefix, strlen(p->pri->internationalprefix)) == 0) {
@@ -4604,35 +4606,7 @@
 	return res;
 }
 
-#ifdef ZAPATA_PRI
-static void set_calltype(struct ast_channel *chan, int ctype)
-{
-	char *s = "UNKNOWN";
-	switch(ctype) {
-	case PRI_TRANS_CAP_SPEECH:
-		s = "SPEECH";
-		break;
-	case PRI_TRANS_CAP_DIGITAL:
-		s = "DIGITAL";
-		break;
-	case PRI_TRANS_CAP_RESTRICTED_DIGITAL:
-		s = "RESTRICTED_DIGITAL";
-		break;
-	case PRI_TRANS_CAP_3_1K_AUDIO:
-		s = "31KAUDIO";
-		break;
-	case PRI_TRANS_CAP_7K_AUDIO:
-		s = "7KAUDIO";
-		break;
-	case PRI_TRANS_CAP_VIDEO:
-		s = "VIDEO";
-		break;
-	}
-	pbx_builtin_setvar_helper(chan, "CALLTYPE", s);
-}
-#endif
-
-static struct ast_channel *zt_new(struct zt_pvt *i, int state, int startpbx, int index, int law, int ctype)
+static struct ast_channel *zt_new(struct zt_pvt *i, int state, int startpbx, int index, int law, int transfercapability)
 {
 	struct ast_channel *tmp;
 	int deflaw;
@@ -4764,14 +4738,15 @@
 		tmp->cid.cid_pres = i->callingpres;
 		tmp->cid.cid_ton = i->cid_ton;
 #ifdef ZAPATA_PRI
-		set_calltype(tmp, ctype);
-		/* Assume calls are not idle calls unless we're told differently */
-		i->isidlecall = 0;
-		i->alreadyhungup = 0;
-		if (ctype & PRI_TRANS_CAP_DIGITAL) {
+		tmp->transfercapability = transfercapability;
+		pbx_builtin_setvar_helper(tmp, "TRANSFERCAPABILITY", ast_transfercapability2str(transfercapability));
+		if (transfercapability & PRI_TRANS_CAP_DIGITAL) {
 			i->digital = 1;
 			ast_set_flag(tmp, AST_FLAG_DIGITAL);
 		}
+		/* Assume calls are not idle calls unless we're told differently */
+		i->isidlecall = 0;
+		i->alreadyhungup = 0;
 #endif
 		/* clear the fake event in case we posted one before we had ast_chanenl */
 		i->fake_event = 0;
Index: include/asterisk/channel.h
===================================================================
RCS file: /usr/cvsroot/asterisk/include/asterisk/channel.h,v
retrieving revision 1.80
diff -u -r1.80 channel.h
--- include/asterisk/channel.h	28 Mar 2005 20:48:24 -0000	1.80
+++ include/asterisk/channel.h	1 Apr 2005 09:05:13 -0000
@@ -315,6 +315,9 @@
 	/*! channel flags of AST_FLAG_ type */
 	unsigned int flags;
 	
+	/* ISDN Transfer Capability - AST_FLAG_DIGITAL is simply not enough */
+	unsigned short transfercapability;
+	
 	struct ast_frame *readq;
 	int alertpipe[2];
 	/*! Write translation path */
@@ -447,6 +450,8 @@
 #define AST_DEVICE_INVALID	4
 /*! Device is unavailable */
 #define AST_DEVICE_UNAVAILABLE	5
+/*! Device is ringing */
+#define AST_DEVICE_RINGING	6
 
 /*! Create a channel structure */
 /*! Returns NULL on failure to allocate */
@@ -810,6 +815,11 @@
  * Returns the text form of the binary state given
  */
 char *ast_state2str(int state);
+
+
+/*! Return the string from a given transfercapability */
+char *ast_transfercapability2str(int transfercapability);
+
 
 /* Options: Some low-level drivers may implement "options" allowing fine tuning of the
    low level channel.  See frame.h for options.  Note that many channel drivers may support
Index: apps/app_settransfercapability.c
===================================================================
RCS file: apps/app_settransfercapability.c
diff -N apps/app_settransfercapability.c
--- /dev/null	1 Jan 1970 00:00:00 -0000
+++ apps/app_settransfercapability.c	1 Jan 1970 00:00:00 -0000
@@ -0,0 +1,114 @@
+/*
+ * Asterisk -- A telephony toolkit for Linux.
+ *
+ * App to set the ISDN Transfer Capability
+ * 
+ * Copyright (C) 2005, Frank Sautter, levigo holding gmbh, www.levigo.de
+ *
+ * Frank Sautter - asterisk+at+sautter+dot+com 
+ *
+ * This program is free software, distributed under the terms of
+ * the GNU General Public License
+ */
+ 
+#include <asterisk/logger.h>
+#include <asterisk/channel.h>
+#include <asterisk/pbx.h>
+#include <asterisk/module.h>
+#include <asterisk/options.h>
+#include <string.h>
+#include <stdlib.h>
+#include <libpri.h>
+
+
+static char *app = "SetTransferCapability";
+
+static char *synopsis = "Set ISDN Transfer Capability";
+
+STANDARD_LOCAL_USER;
+
+LOCAL_USER_DECL;
+
+static struct {	int val; char *name; } transcaps[] = {
+	{ PRI_TRANS_CAP_SPEECH,				"SPEECH" },
+	{ PRI_TRANS_CAP_DIGITAL,			"DIGITAL" },
+	{ PRI_TRANS_CAP_RESTRICTED_DIGITAL,	"RESTRICTED_DIGITAL" },
+	{ PRI_TRANS_CAP_3_1K_AUDIO,			"3K1AUDIO" },
+	{ PRI_TRANS_CAP_DIGITAL_W_TONES,	"DIGITAL_W_TONES" },
+	{ PRI_TRANS_CAP_VIDEO,				"VIDEO" },
+};
+
+static char *descrip = 
+"  SetTransferCapability(transfercapability): Set the ISDN Transfer \n"
+"Capability of a call to a new value.\n"
+"Always returns 0.  Valid Transfer Capabilities are:\n"
+"\n"
+"  SPEECH             : 0x00 - Speech (default, voice calls)\n"
+"  DIGITAL            : 0x08 - Unrestricted digital information (data calls)\n"
+"  RESTRICTED_DIGITAL : 0x09 - Restricted digital information\n"
+"  3K1AUDIO           : 0x10 - 3.1kHz Audio (fax calls)\n"
+"  DIGITAL_W_TONES    : 0x11 - Unrestricted digital information with tones/announcements\n"
+"  VIDEO              : 0x18 - Video:\n"
+"\n"
+;
+
+static int settransfercapability_exec(struct ast_channel *chan, void *data)
+{
+	char tmp[256] = "";
+	struct localuser *u;
+	int x;
+	char *opts;
+	int transfercapability = -1;
+	
+	if (data)
+		strncpy(tmp, (char *)data, sizeof(tmp) - 1);
+	opts = strchr(tmp, '|');
+	if (opts)
+		*opts = '\0';
+	for (x=0;x<sizeof(transcaps) / sizeof(transcaps[0]);x++) {
+		if (!strcasecmp(transcaps[x].name, tmp)) {
+			transfercapability = transcaps[x].val;
+			break;
+		}
+	}
+	if (transfercapability < 0) {
+		ast_log(LOG_WARNING, "'%s' is not a valid transfer capability (see 'show application SetTransferCapability')\n", tmp);
+		return 0;
+	} else {
+		LOCAL_USER_ADD(u);
+		chan->transfercapability = (unsigned short)transfercapability;
+		LOCAL_USER_REMOVE(u);
+		if (option_verbose > 2)
+			ast_verbose(VERBOSE_PREFIX_3 "Setting transfer capability to: 0x%.2x - %s.\n", transfercapability, tmp);			
+		return 0;
+	}
+}
+
+
+int unload_module(void)
+{
+	STANDARD_HANGUP_LOCALUSERS;
+	return ast_unregister_application(app);
+}
+
+int load_module(void)
+{
+	return ast_register_application(app, settransfercapability_exec, synopsis, descrip);
+}
+
+char *description(void)
+{
+	return descrip;
+}
+
+int usecount(void)
+{
+	int res;
+	STANDARD_USECOUNT(res);
+	return res;
+}
+
+char *key()
+{
+	return ASTERISK_GPL_KEY;
+}
